<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace FederationCLI;

    use Exception;
    use FederationLib\Classes\Configuration;
    use FederationLib\Enums\Standard\Methods;
    use FederationLib\Exceptions\DatabaseException;
    use FederationLib\Exceptions\Standard\AccessDeniedException;
    use FederationLib\Exceptions\Standard\ClientNotFoundException;
    use FederationLib\Exceptions\Standard\InternalServerException;
    use FederationLib\Exceptions\Standard\InvalidClientDescriptionException;
    use FederationLib\Exceptions\Standard\InvalidClientNameException;
    use FederationLib\FederationLib;
    use JsonException;
    use OptsLib\Parse;
    use TamerLib\Enums\TamerMode;
    use TamerLib\Exceptions\ServerException;
    use TamerLib\Exceptions\WorkerFailedException;
    use TamerLib\tm;

    class InteractiveMode
    {
        /**
         * @var FederationLib
         */
        private static $federation_lib;

        /**
         * Main entry point for the interactive mode
         *
         * @param array $args
         * @return void
         * @throws ServerException
         * @throws WorkerFailedException
         */
        public static function main(array $args=[]): void
        {
            tm::initialize(TamerMode::CLIENT);
            tm::createWorker(Configuration::getTamerLibConfiguration()->getCliWorkers(), FederationLib::getSubprocessPath());

            self::$federation_lib = new FederationLib();

            while(true)
            {
                print(sprintf('%s@%s:~$ ', 'root', Configuration::getHostName()));
                $input = trim(fgets(STDIN));

                try
                {
                    switch(strtolower(explode(' ', $input)[0]))
                    {
                        case Methods::PING:
                            self::ping();
                            break;
                        case Methods::WHOAMI:
                            self::whoami();
                            break;

                        case Methods::CREATE_CLIENT:
                            self::createClient($input);
                            break;
                        case Methods::GET_CLIENT:
                            self::getClient($input);
                            break;
                        case Methods::CHANGE_CLIENT_NAME:
                            self::changeClientName($input);
                            break;

                        default:
                            print(sprintf('Command \'%s\' not found' . PHP_EOL, $input));
                            break;
                    }
                }
                catch(Exception $e)
                {
                    print(sprintf('Error: %s' . PHP_EOL, $e->getMessage()));
                }
            }
        }

        /**
         * Invokes the ping method
         *
         * @return void
         * @throws AccessDeniedException
         * @throws ClientNotFoundException
         * @throws InternalServerException
         */
        private static function ping(): void
        {
            if(self::$federation_lib->ping(null))
            {
                print('OK' . PHP_EOL);
                return;
            }

            print('ERROR' . PHP_EOL);
        }

        /**
         * Invokes the whoami method and prints the result
         *
         * @return void
         * @throws AccessDeniedException
         * @throws ClientNotFoundException
         * @throws InternalServerException
         */
        private static function whoami(): void
        {
            print(self::$federation_lib->whoami(null) . PHP_EOL);
        }

        /**
         * @param string $input
         * @return void
         * @throws AccessDeniedException
         * @throws ClientNotFoundException
         * @throws DatabaseException
         * @throws InternalServerException
         * @throws InvalidClientDescriptionException
         * @throws InvalidClientNameException
         */
        private static function createClient(string $input): void
        {
            $parsed_arguments = Parse::parseArgument($input);

            $name = $parsed_arguments['name'] ?? $parsed_arguments['n'] ?? null;
            $description = $parsed_arguments['description'] ?? $parsed_arguments['d'] ?? null;

            print(self::$federation_lib->createClient(null, $name, $description) . PHP_EOL);
        }

        /**
         * @param string $input
         * @return void
         * @throws AccessDeniedException
         * @throws DatabaseException
         * @throws InternalServerException
         * @throws JsonException
         * @noinspection PhpMultipleClassDeclarationsInspection
         */
        private static function getClient(string $input): void
        {
            $parsed_arguments = Parse::parseArgument($input);

            $uuid = $parsed_arguments['uuid'] ?? $parsed_arguments['u'] ?? null;
            $raw = $parsed_arguments['raw'] ?? $parsed_arguments['r'] ?? false;

            if($uuid === null | $uuid === '')
            {
                print('Missing required argument \'uuid\'' . PHP_EOL);
                return;
            }

            try
            {
                $client = self::$federation_lib->getClient(null, $uuid);

                if($raw)
                {
                    print(json_encode($client->toArray(), JSON_THROW_ON_ERROR | JSON_PRETTY_PRINT) . PHP_EOL);
                    return;
                }

                Utilities::printData('CLIENT LOOKUP RESULTS', [
                    'UUID' => $client->getUuid(),
                    'NAME' => $client->getName(),
                    'DESCRIPTION' => $client->getDescription() ?? 'N/A',
                    'PERMISSION_ROLE' => $client->getPermissionRole(),
                    'CREATED_AT' => date('Y-m-d H:i:s', $client->getCreatedTimestamp()),
                    'UPDATED_AT' => date('Y-m-d H:i:s', $client->getUpdatedTimestamp()),
                    'SEEN_AT' => date('Y-m-d H:i:s', $client->getSeenTimestamp())
                ]);
            }
            catch(ClientNotFoundException)
            {
                print(sprintf('Client with UUID \'%s\' not found' . PHP_EOL, $uuid));
            }
        }

        /**
         * Changes the name of a client with the given UUID
         *
         * @param string $input
         * @return void
         * @throws AccessDeniedException
         * @throws DatabaseException
         * @throws InternalServerException
         * @throws InvalidClientNameException
         */
        private static function changeClientName(string $input): void
        {
            $parsed_arguments = Parse::parseArgument($input);

            $uuid = $parsed_arguments['uuid'] ?? $parsed_arguments['u'] ?? null;
            $name = $parsed_arguments['name'] ?? $parsed_arguments['n'] ?? null;

            if($uuid === null | $uuid === '')
            {
                print('Missing required argument \'uuid\'' . PHP_EOL);
                return;
            }

            if($name === null | $name === '')
            {
                print('Missing required argument \'name\'' . PHP_EOL);
                return;
            }

            try
            {
                self::$federation_lib->changeClientName(null, $uuid, $name);
                print('OK' . PHP_EOL);
            }
            catch(ClientNotFoundException)
            {
                print(sprintf('Client with UUID \'%s\' not found' . PHP_EOL, $uuid));
            }
        }
    }