<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace FederationLib\Classes;

    use Exception;
    use FederationLib\Classes\Configuration\CacheServerConfiguration;
    use FederationLib\Enums\Misc;
    use FederationLib\Exceptions\CacheConnectionException;
    use FederationLib\Exceptions\CacheDriverException;
    use LogLib\Log;
    use RedisException;

    class Redis
    {
        /**
         * @var int|null
         */
        private $redis_last_connection_time;

        /**
         * @var \Redis|null
         */
        private $redis_connection;

        /**
         * @var CacheServerConfiguration
         */
        private $configuration;

        /**
         * @var bool
         */
        private $connection_error;

        /**
         * Redis constructor.
         *
         * @param CacheServerConfiguration $configuration
         */
        public function __construct(CacheServerConfiguration $configuration)
        {
            $this->configuration = $configuration;
            $this->connection_error = false;
        }

        /**
         * Indicates if the redis server is available to connect to
         *
         * @return bool
         */
        public function isAvailable(): bool
        {
            if(!$this->configuration->isEnabled())
            {
                return false;
            }

            return true;
        }

        /**
         * Determines if the redis server is connected.
         *
         * @return bool
         */
        public function isConnected(): bool
        {
            if(!$this->configuration->isEnabled())
            {
                return false;
            }

            return $this->redis_connection !== null;
        }

        /**
         * @return bool
         */
        public function isConnectionError(): bool
        {
            return $this->connection_error;
        }

        /**
         * Disconnects from the redis server if it's connected.
         *
         * @param bool $reset_error
         * @return void
         */
        public function disconnect(bool $reset_error=true): void
        {
            if(!$this->isConnected())
            {
                return;
            }

            try
            {
                $this->redis_connection->close();
            }
            catch(Exception $e)
            {
                Log::warning(Misc::FEDERATIONLIB, sprintf('Failed to disconnect from redis server: %s', $e->getMessage()));
            }

            if($reset_error)
            {
                $this->connection_error = false;
            }

            $this->redis_connection = null;
        }

        /**
         * Establishes a connection to the redis server.
         *
         * @param bool $throw_exception
         * @return void
         * @throws CacheConnectionException
         * @throws CacheDriverException
         */
        public function connect(bool $throw_exception=true): void
        {
            if(!$this->configuration->isEnabled())
            {
                if($throw_exception)
                {
                    throw new CacheDriverException(sprintf('Failed to connect to the redis server \'%s\' because it\'s disabled.', $this->configuration->getName()));
                }
            }

            if($this->redis_connection !== null)
            {
                if($this->redis_last_connection_time === null)
                {
                    return;
                }

                if($this->redis_last_connection_time < (time() - $this->configuration->getReconnectInterval()))
                {
                    Log::verbose(Misc::FEDERATIONLIB, sprintf('Interval limit of %s seconds to reconnect to the redis server \'%s\' has been reached, reconnecting...', $this->configuration->getReconnectInterval(), $this->configuration->getName()));
                    $this->disconnect();
                }
                else
                {
                    return;
                }
            }

            try
            {
                Log::info(Misc::FEDERATIONLIB, sprintf('Connecting to the redis server \'%s\': %s:%s', $this->configuration->getName(), $this->configuration->getHost(), $this->configuration->getPort()));
                $redis = new \Redis();
                $redis->connect($this->configuration->getHost(), $this->configuration->getPort());
                if($this->configuration->getPassword() !== null)
                {
                    $redis->auth($this->configuration->getPassword());
                }
                $redis->select($this->configuration->getDatabase());
            }
            catch(Exception $e)
            {
                $this->connection_error = true;

                if($throw_exception)
                {
                    throw new CacheConnectionException(sprintf('Failed to connect to the redis server \'%s\': %s', $this->configuration->getName(), $e->getMessage()), $e->getCode(), $e);
                }

                Log::warning(Misc::FEDERATIONLIB, sprintf('Failed to connect to the redis server \'%s\': %s', $this->configuration->getName(), $e->getMessage()));
                return;
            }

            $this->redis_connection = $redis;
            $this->redis_last_connection_time = time();
        }

        /**
         * Returns/Establishes a connection to the redis server.
         * Returns null if redis is disabled.
         *
         * @param bool $throw_exception
         * @return \Redis|null
         * @throws CacheConnectionException
         * @throws CacheDriverException
         */
        public function getConnection(bool $throw_exception=true): ?\Redis
        {
            if(!$this->configuration->isEnabled())
            {
                return null;
            }

            if(!$this->isConnected())
            {
                $this->connect($throw_exception);
            }
            else
            {
                if($this->redis_last_connection_time < (time() - $this->configuration->getReconnectInterval()))
                {
                    $this->connect($throw_exception);
                }
            }

            return $this->redis_connection;
        }

        /**
         * Returns the last time the redis server was connected to.
         *
         * @return int|null
         */
        public function getRedisLastConnectionTime(): ?int
        {
            return $this->redis_last_connection_time;
        }

        /**
         * @return CacheServerConfiguration
         */
        public function getConfiguration(): CacheServerConfiguration
        {
            return $this->configuration;
        }

    }