<?php

    namespace FederationLib\Interfaces;

    use FederationLib\Classes\Configuration\CacheServerConfiguration;
    use Memcached;
    use Redis;

    interface CacheDriverInterface
    {
        /**
         * Constructs a cache server driver
         *
         * @param CacheServerConfiguration $configuration
         */
        public function __construct(CacheServerConfiguration $configuration);

        /**
         * Returns the configuration for the driver
         *
         * @return CacheServerConfiguration
         */
        public function getConfiguration(): CacheServerConfiguration;

        /**
         * Returns the driver connection instance.
         *
         * @return Redis|Memcached
         */
        public function getConnection(): Redis|Memcached;

        /**
         * Returns the values of the specified key
         *
         * For every key that does not hold a string value or does not exist, the special value false is returned.
         * Because of this, the operation never fails.
         *
         * @param string $key
         * @return mixed
         */
        public function get(string $key): mixed;

        /**
         * Gets a value from the hash stored at key. If the hash table doesn't exist,
         * or the key doesn't exist, FALSE is returned
         *
         * @param string $key
         * @param string $field
         * @return mixed
         */
        public function hGet(string $key, string $hashKey): mixed;

        /**
         * Returns the whole hash, as an array of strings indexed by string
         *
         * @param string $key
         * @return array
         */
        public function hGetAll(string $key): array;

        /**
         * Sets a value in the cache server
         *
         * @param string $key
         * @param mixed $value
         * @return void
         */
        public function set(string $key, mixed $value): void;

        /**
         * Fills in a whole hash. Non-string values are converted to string, using the standard (string) cast.
         * NULL values are stored as empty string
         *
         * @param string $key
         * @param array $values
         * @return void
         */
        public function hMSet(string $key, array $values): void;

        /**
         * Adds a value to the hash stored at key. If this value is already in the hash, FALSE is returned.
         *
         * @param string $key
         * @param string $hash_key
         * @param mixed $value
         * @return void
         */
        public function hSet(string $key, string $hash_key, mixed $value): void;

        /**
         * Verify if the specified key exists
         *
         * @param string $key
         * @return bool
         */
        public function exists($key): bool;

        /**
         * Remove specified keys
         *
         * @param $key1
         * @param mixed ...$other_keys
         * @return void
         */
        public function delete($key1, ...$other_keys): void;

        /**
         * Sets a key to expire in a certain amount of seconds
         *
         * @param string $key
         * @param int $seconds
         * @return void
         */
        public function expire(string $key, int $seconds): void;

        /**
         * Determines if the cache server is available (does not necessarily mean that the server is connected)
         * This is useful for checking if the cache server is available before attempting to use it
         *
         * @return bool
         */
        public function isAvailable(): bool;

        /**
         * Determines if the cache server is currently connected
         * If the ping parameter is set to true, this will ping the server to ensure that it is connected
         * otherwise it will assume the connection is stable if a connection has been established
         *
         * @param bool $ping This will ping the server to check if it is connected
         * @return bool
         */
        public function isConnected(bool $ping=false): bool;

        /**
         * Connects to the cache server
         *
         * @return void
         */
        public function connect(): void;

        /**
         * Disconnects from the cache server
         *
         * @return void
         */
        public function disconnect(): void;

    }