<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace FederationLib\Objects;

    use FederationCLI\Utilities;
    use FederationLib\Classes\Security;
    use FederationLib\Enums\Standard\PermissionRole;
    use FederationLib\Interfaces\SerializableObjectInterface;

    class Client implements SerializableObjectInterface
    {
        /**
         * @var string
         */
        private $uuid;

        /**
         * @var bool
         */
        private $enabled;

        /**
         * @var string|null
         */
        private $name;

        /**
         * @var string|null
         */
        private $description;

        /**
         * @var string|null
         */
        private $secret_totp;

        /**
         * @var int
         * @see PermissionRole
         */
        private $permission_role;

        /**
         * @var string[]
         */
        private $flags;

        /**
         * @var int
         */
        private $created_timestamp;

        /**
         * @var int
         */
        private $updated_timestamp;

        /**
         * @var int
         */
        private $seen_timestamp;

        /**
         * Client constructor.
         */
        public function __construct()
        {
            $this->enabled = true;
            $this->flags = [];
            $this->permission_role = PermissionRole::CLIENT;
        }

        /**
         * Returns the client's unique ID.
         *
         * @return string
         */
        public function getUuid(): string
        {
            return $this->uuid;
        }

        /**
         * @return bool
         */
        public function isEnabled(): bool
        {
            return $this->enabled;
        }

        /**
         * Returns the client's name.
         *
         * @return string
         */
        public function getName(): string
        {
            return $this->name;
        }

        /**
         * Optional. Returns the client's description.
         *
         * @return string|null
         */
        public function getDescription(): ?string
        {
            return $this->description;
        }

        /**
         * Indicates whether or not the client requires authentication.
         *
         * @return bool
         */
        public function requiresAuthentication(): bool
        {
            return $this->secret_totp !== null;
        }

        /**
         * Validates the given authentication code against the client's secret.
         *
         * @param string|null $authentication_code
         * @param int|null $timestamp
         * @return bool
         */
        public function validateAuthentication(?string $authentication_code=null, ?int $timestamp=null): bool
        {
            if($this->secret_totp === null)
            {
                // Always authenticate if authentication is disabled.
                return true;
            }

            if($authentication_code === null)
            {
                // Authentication code not provided but required.
                return false;
            }

            // Authentication Code: sha1(client_id + totp_code)
            return hash('sha1', $this->uuid . Security::generateCode($this->secret_totp, $timestamp)) === $authentication_code;
        }

        /**
         * Returns the client's query permission level.
         *
         * @return int
         */
        public function getPermissionRole(): int
        {
            return $this->permission_role;
        }

        /**
         * Returns the client's flags.
         *
         * @return string[]
         */
        public function getFlags(): array
        {
            return $this->flags;
        }

        /**
         * Returns True if the client has the given flag.
         *
         * @param string $flag
         * @return bool
         */
        public function hasFlag(string $flag): bool
        {
            return in_array($flag, $this->flags);
        }

        /**
         * Returns the Unix Timestamp for when the client was last created on the server
         *
         * @return int
         */
        public function getCreatedTimestamp(): int
        {
            return $this->created_timestamp;
        }

        /**
         * Returns the Unix Timestamp for when the client was last updated on the server.
         *
         * @return int
         */
        public function getUpdatedTimestamp(): int
        {
            return $this->updated_timestamp;
        }

        /**
         * Returns the Unix Timestamp for when the client was last seen by the server.
         *
         * @return int
         */
        public function getSeenTimestamp(): int
        {
            return $this->seen_timestamp;
        }

        /**
         * Returns an array representation of the object
         *
         * @return array
         */
        public function toArray(): array
        {
            $flags = null;

            if($this->flags !== null && count($this->flags) > 0)
            {
                $flags = implode(',', $this->flags);
            }

            return [
                'uuid' => $this->uuid,
                'enabled' => $this->enabled,
                'name' => $this->name,
                'description' => $this->description,
                'secret_totp' => $this->secret_totp,
                'permission_role' => $this->permission_role,
                'flags' => $flags,
                'created_timestamp' => $this->created_timestamp,
                'updated_timestamp' => $this->updated_timestamp,
                'seen_timestamp' => $this->seen_timestamp,
            ];
        }

        /**
         * Constructs object from an array representation.
         *
         * @param array $array
         * @return Client
         */
        public static function fromArray(array $array): Client
        {
            $client = new self();

            $client->uuid = $array['uuid'] ?? null;
            $client->enabled = Utilities::parseBoolean($array['enabled']);
            $client->name = $array['name'] ?? null;
            $client->description = $array['description'] ?? null;
            $client->secret_totp = $array['secret_totp'] ?? null;
            $client->permission_role = $array['permission_role'] ?? PermissionRole::CLIENT;
            if(isset($array['flags']))
            {
                $client->flags = explode(',', $array['flags']);
            }
            else
            {
                $client->flags = [];
            }
            $client->created_timestamp = $array['created_timestamp'] ?? 0;
            $client->updated_timestamp = $array['updated_timestamp'] ?? 0;
            $client->seen_timestamp = $array['seen_timestamp'] ?? 0;

            return $client;
        }

        /**
         * Returns the object identifier.
         *
         * @return string
         */
        public function __toString(): string
        {
            return sprintf('Client<%s>', $this->uuid ?? spl_object_hash($this));
        }
    }